require "util"
require "defines"
require "story"

-- localisation shortcut
gt = game.gettext

game.oninit(function()
  game.player.force.disableallprototypes()
  game.mapsettings.pollution.enabled = false
  game.evolutionfactor = 0
  game.mapsettings.enemy_evolution.enabled = false

  local recipelist = game.player.force.recipes
  recipelist["iron-plate"].enabled = true
  recipelist["copper-plate"].enabled = true
  recipelist["stone-brick"].enabled = true
  recipelist["wall"].enabled = true
  recipelist["wood"].enabled = true
  recipelist["stone-furnace"].enabled = true
  recipelist["iron-stick"].enabled = true
  recipelist["iron-axe"].enabled = true
  recipelist["wooden-chest"].enabled = true
  recipelist["iron-gear-wheel"].enabled = true
  recipelist["burner-mining-drill"].enabled = true
  recipelist["basic-transport-belt"].enabled = true
  recipelist["burner-inserter"].enabled = true
  recipelist["pump"].enabled = true
  recipelist["pipe"].enabled = true
  recipelist["pipe-to-ground"].enabled = true
  recipelist["boiler"].enabled = true
  recipelist["steam-engine"].enabled = true
  recipelist["electronic-circuit"].enabled = true
  recipelist["copper-cable"].enabled = true
  recipelist["pistol"].enabled = true
  recipelist["submachine-gun"].enabled = true
  recipelist["basic-bullet-magazine"].enabled = true
  recipelist["basic-mining-drill"].enabled = true
  recipelist["basic-inserter"].enabled = true
  recipelist["small-electric-pole"].enabled = true
  recipelist["lab"].enabled = true
  recipelist["science-pack-1"].enabled = true
  recipelist["science-pack-2"].enabled = true
  local technologylist = game.player.force.technologies
  technologylist["iron-working"].enabled = true
  technologylist["steel-processing"].enabled = true
  technologylist["optics"].enabled = true
  technologylist["logistics"].enabled = true
  technologylist["logistics-2"].enabled = true
  technologylist["automation"].enabled = true
  technologylist["automation-2"].enabled = true
  technologylist["chemistry"].enabled = true
  technologylist["flammables"].enabled = true
  technologylist["explosives"].enabled = true
  technologylist["flame-thrower"].enabled = true
  technologylist["electronics"].enabled = true
  technologylist["rocketry"].enabled = true
  technologylist["military"].enabled = true
  technologylist["military-2"].enabled = true
  technologylist["turrets"].enabled = true
  technologylist["armor-making"].enabled = true
  technologylist["bullet-damage-1"].enabled = true
  technologylist["bullet-damage-2"].enabled = true
  technologylist["bullet-damage-3"].enabled = true
  technologylist["bullet-speed-1"].enabled = true
  technologylist["bullet-speed-2"].enabled = true
  technologylist["bullet-speed-3"].enabled = true

  local character = game.player.character
  character.insert{name = "basic-transport-belt", count = 20}
  character.insert{name = "basic-inserter", count = 15}
  character.insert{name = "small-electric-pole", count = 10}
  character.insert{name = "basic-mining-drill", count = 5}
  character.insert{name = "steam-engine", count = 3}
  character.insert{name = "boiler", count = 5}
  character.insert{name = "stone-furnace", count = 5}
  character.insert{name = "raw-wood", count = 25}
  character.insert{name = "iron-plate", count = 250}
  character.insert{name = "copper-plate", count = 200}
  character.insert{name = "coal", count = 200}
  character.insert{name = "submachine-gun", count = 1}
  character.insert{name = "basic-bullet-magazine", count = 20}
  character.insert{name = "iron-axe", count = 1}
end)

storytable =
{
  {
    {
      init =
      function()
        if game.difficulty == defines.difficulty.normal then
          glob.timelimit = 100*60*60
        elseif game.difficulty == defines.difficulty.hard then
          glob.timelimit = 85*60*60
        end
      end,
      condition = story.Story:elapsedcheck(2),
      action = 
      function()
        game.showmessagedialog(gt("msg-wreck-location"))
      end
    },
    {
      action = 
      function()
        game.showmessagedialog(gt("msg-wreck-motivation"))
      end
    },
    {
      action =
      function()
        game.showmessagedialog(gt("msg-wreck-guarded"))
      end
    },
    {
      action =
      function()
        if glob.timelimit ~= nil then
          game.showmessagedialog(gt("msg-wreck-time-limit", (glob.timelimit / 3600)))
        end
      end
    },
    {
      action =
      function()
        game.showmessagedialog(gt("msg-tip"))
      end
    },
    {
      action =
      function()
        game.showmessagedialog(gt("msg-research"))
      end
    },
    -- post tutorial game state
    {
      init =
      function(event)
        if glob.timelimit == nil then
          game.player.setgoaldescription(gt("goal-get-computer-without-time"))
        end
      end,
      update = 
      function(event)
        
        -- Helper message that the player needs rockets after 10 minutes of playing
        if event.name == defines.events.ontick and
           event.tick - mainstory.context.currentstorystartedat == 60*60*10 and
           glob.rocketmessageshown == nil then
          game.showmessagedialog(gt("msg-rockets-needed"))
          glob.rocketmessageshown = true
        end
        
        -- Helper message that the player needs armor (if he doesn't have it so far) after 15 minutes of playing
        if event.name == defines.events.ontick and
           event.tick - mainstory.context.currentstorystartedat == 60*60*15 and
           glob.armormessageshown == nil and
           game.player.character.getitemcount("basic-armor") == 0 then
          game.showmessagedialog(gt("msg-armor-needed"))
          glob.armormessageshown = true
        end
        
        -- Check the time limit (glob.timelimit is not specified, no time limit)
        if glob.timelimit ~= nil then
          local timeleft = (glob.timelimit - event.tick)
          if timeleft > 0 then
            game.player.setgoaldescription(gt("goal-get-computer", util.formattime(timeleft)), true)
          else
            game.setgamestate{gamefinished=true, playerwon=false}
          end
        end
      end,
      condition =
      function()
        return game.player.character ~= nil and game.player.character.getitemcount("computer") > 0
      end,
      action =
      function()
      end
    },
  },
}

mainstory = story.Story:new(storytable)

game.onsave(function()
  glob.storydata = mainstory:export()
end)

game.onload(function()
  mainstory:import(glob.storydata)
end)

function manageattacks(tick)
  if glob.lastattackat == nil then
    glob.lastattackat = 0
  end
  if glob.attackcount == nil then
    glob.attackcount = 3
  end
  -- attack rate is once per 90 seconds, it doesn't change
  if glob.attackfrequency == nil then
    glob.attackfrequency = 60 * 3
  end
  if tick - glob.lastattackat > 60 * glob.attackfrequency then
    glob.lastattackat = tick
    game.setmulticommand({type=defines.command.attack,
                          target=game.player.character,
                          distraction=defines.distraction.byenemy},
                          glob.attackcount)
    -- every attack is 0.6 aliens bigge
    glob.attackcount = glob.attackcount + 0.6
  end
end

game.onevent(defines.events, function(event)
  manageattacks(event.tick)
  mainstory:update(event, "level-02")
end)
